@@ -5,10 +5,10 @@ from typing import TYPE_CHECKING, Any, Literal, Unpack, overload
 from urllib.parse import urljoin
 from webbrowser import open as open_browser
 
-from loguru import logger
 from notte_core.actions import BaseAction
 from notte_core.browser.observation import ExecutionResult
 from notte_core.common.config import CookieDict, PerceptionType, config
+from notte_core.common.logging import logger
 from notte_core.common.resource import SyncResource
 from notte_core.common.telemetry import track_usage
 from notte_core.data.space import ImageData, StructuredData, TBaseModel
@@ -47,6 +47,29 @@ from notte_sdk.websockets.jupyter import display_image_in_notebook
 if TYPE_CHECKING:
     from notte_sdk.client import NotteClient
 
+_playwright_available = False
+_async_playwright_available = False
+
+try:
+    from playwright.sync_api import Browser as BrowserSync
+    from playwright.sync_api import Page as PageSync
+    from playwright.sync_api import Playwright as PlaywrightSync
+    from playwright.sync_api import sync_playwright as _sync_playwright
+
+    _playwright_available = True
+except ImportError:
+    _sync_playwright = None
+
+try:
+    from playwright.async_api import Browser as BrowserAsync
+    from playwright.async_api import Page as PageAsync
+    from playwright.async_api import Playwright as PlaywrightAsync
+    from playwright.async_api import async_playwright as _async_playwright
+
+    _async_playwright_available = True
+except ImportError:
+    _async_playwright = None
+
 
 class SessionViewerType(StrEnum):
     CDP = "cdp"
@@ -407,13 +430,13 @@ class SessionsClient(BaseClient):
     @track_usage("cloud.session.replay")
     def replay(self, session_id: str) -> MP4Replay:
         """
-        Downloads the replay for the specified session in webp format.
+        Downloads the replay for the specified session in mp4 format.
 
         Args:
             session_id: The identifier of the session to download the replay for.
 
         Returns:
-            WebpReplay: The replay file in webp format.
+            MP4Replay: The replay file in mp4 format.
         """
         endpoint = SessionsClient._session_debug_replay_endpoint(session_id=session_id)
         file_bytes = self._request_file(endpoint, file_type="mp4")
@@ -549,6 +572,14 @@ class RemoteSession(SyncResource):
         self.default_perception_type: PerceptionType = perception_type
         self.default_raise_on_failure: bool = raise_on_failure
         self._cookie_file: Path | None = Path(cookie_file) if cookie_file is not None else None
+        # Sync playwright instances
+        self._playwright_context: "PlaywrightSync | None" = None
+        self._playwright_browser: "BrowserSync | None" = None
+        self._playwright_page: Any | None = None
+        # Async playwright instances
+        self._async_playwright_context: "PlaywrightAsync | None" = None
+        self._async_playwright_browser: "BrowserAsync | None" = None
+        self._async_playwright_page: "PageAsync | None" = None
 
         if self.storage is not None and not self.request.use_file_storage:
             logger.warning(
@@ -562,6 +593,49 @@ class RemoteSession(SyncResource):
     ) -> None:
         if exc_val is not None:  # pyright: ignore [reportUnnecessaryComparison]
             logger.warning(f"Session exiting because of exception: {exc_val}")
+
+        # Clean up sync playwright resources
+        if self._playwright_browser is not None:
+            self._playwright_browser.close()
+            self._playwright_browser = None
+        if self._playwright_context is not None:
+            self._playwright_context.stop()
+            self._playwright_context = None
+        self._playwright_page = None
+
+        self.stop()
+
+        if isinstance(exc_val, KeyboardInterrupt):
+            raise KeyboardInterrupt() from None
+
+    async def __aenter__(self) -> "RemoteSession":
+        """
+        Async context manager entry point.
+
+        Returns:
+            RemoteSession: The session instance.
+        """
+        self.start()
+        return self
+
+    async def __aexit__(
+        self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: type[BaseException] | None
+    ) -> None:
+        """
+        Async context manager exit point with cleanup of async playwright resources.
+        """
+        if exc_val is not None:  # pyright: ignore [reportUnnecessaryComparison]
+            logger.warning(f"Session exiting because of exception: {exc_val}")
+
+        # Clean up async playwright resources
+        if self._async_playwright_browser is not None:
+            await self._async_playwright_browser.close()
+            self._async_playwright_browser = None
+        if self._async_playwright_context is not None:
+            await self._async_playwright_context.stop()
+            self._async_playwright_context = None
+        self._async_playwright_page = None
+
         self.stop()
 
         if isinstance(exc_val, KeyboardInterrupt):
@@ -589,7 +663,7 @@ class RemoteSession(SyncResource):
         session.start()
         ```
 
-        > Note that we strongly reccomend using the `with` statement to start and stop the session to avoid any issues with session cleanup.
+        > Note that we strongly recommend using the `with` statement to start and stop the session to avoid any issues with session cleanup.
 
         **Example:**
         ```python
@@ -658,7 +732,7 @@ class RemoteSession(SyncResource):
         session.stop()
         ```
 
-        > Note that we strongly reccomend using the `with` statement to start and stop the session to avoid any issues with session cleanup.
+        > Note that we strongly recommend using the `with` statement to start and stop the session to avoid any issues with session cleanup.
 
         **Example:**
         ```python
@@ -718,11 +792,12 @@ class RemoteSession(SyncResource):
 
     def replay(self) -> MP4Replay:
         """
-        Get a replay of the session's execution in WEBP format.
+        Get a replay of the session's execution in MP4 format.
 
         **Example:**
         ```python
         replay = session.replay()
+        replay.save(f"{session.session_id}_replay.mp4")
         ```
 
         > Note that the replay is only available after the session has been stopped.
@@ -730,7 +805,7 @@ class RemoteSession(SyncResource):
         Also you need to perform at least one action for the replay to be available.
 
         Returns:
-            WebpReplay: The replay data in WEBP format.
+            MP4Replay: The replay data in MP4 format.
 
         Raises:
             ValueError: If the session hasn't been started yet (no session_id available).
@@ -781,7 +856,7 @@ class RemoteSession(SyncResource):
         """
         Get the current status of the session.
 
-        This method is usefull if you want to check if the current session is active or not (or when it has been started/stopped).
+        This method is useful if you want to check if the current session is active or not (or when it has been started/stopped).
 
         **Example:**
         ```python
@@ -882,6 +957,117 @@ class RemoteSession(SyncResource):
         debug = self.debug_info()
         return debug.ws.cdp
 
+    @property
+    def page(self) -> "PageSync":
+        """
+        Get a Playwright page connected to the session via CDP.
+
+        This property provides direct access to the browser page using Playwright's API.
+        The connection is established lazily on first access and cached for subsequent calls.
+
+        **Example:**
+        ```python
+        from notte_sdk import NotteClient
+
+        client = NotteClient()
+        with client.Session() as session:
+            # Access the playwright page
+            page = session.page
+            page.goto("https://www.google.com")
+            screenshot = page.screenshot(path="screenshot.png")
+        ```
+
+        Returns:
+            PlaywrightPage: A Playwright page instance connected to the session.
+
+        Raises:
+            ImportError: If playwright is not installed. Install with `pip install notte-sdk[playwright]`
+            ValueError: If the session hasn't been started yet (no session_id available).
+        """
+        if not _playwright_available:
+            raise ImportError("Playwright not installed. Use `pip install notte-sdk[playwright]` to install it.")
+
+        # Return cached page if already connected
+        if self._playwright_page is not None:
+            return self._playwright_page
+
+        if self._async_playwright_page is not None:
+            raise RuntimeError(
+                "Session Page has been initialized with async playwright. Use `await session.apage()` instead."
+            )
+        try:
+            # Initialize playwright context
+            if self._playwright_context is None:
+                if _sync_playwright is None:
+                    raise RuntimeError("Playwright is not initialized")
+                self._playwright_context = _sync_playwright().start()
+
+            # Connect to browser via CDP
+            if self._playwright_browser is None:
+                cdp_url = self.cdp_url()
+                self._playwright_browser = self._playwright_context.chromium.connect_over_cdp(cdp_url)
+
+            # Get the first page from the first context
+            self._playwright_page = self._playwright_browser.contexts[0].pages[0]
+            return self._playwright_page
+        except Exception as e:
+            raise RuntimeError("Failed to access the playwright page from CDP") from e
+
+    @property
+    async def apage(self) -> "PageAsync":
+        """
+        Get an async Playwright page connected to the session via CDP.
+
+        This method provides direct access to the browser page using Playwright's async API.
+        The connection is established lazily on first access and cached for subsequent calls.
+
+        **Example:**
+        ```python
+        from notte_sdk import NotteClient
+
+        client = NotteClient()
+        async with client.Session() as session:
+            # Access the async playwright page
+            page = await session.apage()
+            await page.goto("https://www.google.com")
+            screenshot = await page.screenshot(path="screenshot.png")
+        ```
+
+        Returns:
+            PlaywrightPage: An async Playwright page instance connected to the session.
+
+        Raises:
+            ImportError: If playwright is not installed. Install with `pip install notte-sdk[playwright]`
+            ValueError: If the session hasn't been started yet (no session_id available).
+        """
+        if not _async_playwright_available:
+            raise ImportError("Playwright not installed. Use `pip install notte-sdk[playwright]` to install it.")
+
+        # Return cached page if already connected
+        if self._async_playwright_page is not None:
+            return self._async_playwright_page
+
+        if self._playwright_browser is not None:
+            raise RuntimeError("Session Page has been initialized with sync playwright. Use `session.page` instead.")
+
+        try:
+            # Initialize async playwright context
+            if self._async_playwright_context is None:
+                if _async_playwright is None:
+                    raise RuntimeError("Playwright is not initialized")
+                self._async_playwright_context = await _async_playwright().start()
+
+            # Connect to browser via CDP
+            if self._async_playwright_browser is None:
+                cdp_url = self.cdp_url()
+                self._async_playwright_browser = await self._async_playwright_context.chromium.connect_over_cdp(cdp_url)
+
+            # Get the first page from the first context
+            self._async_playwright_page = self._async_playwright_browser.contexts[0].pages[0]
+            return self._async_playwright_page
+        except Exception as e:
+            raise RuntimeError("Failed to access the async playwright page from CDP") from e
+
     # #######################################################################
     # ############################# PAGE ####################################
     # #######################################################################
@@ -1029,19 +1215,21 @@ class RemoteSession(SyncResource):
         actions either as structured action objects or by specifying action parameters directly.
 
         ```python
+        from notte_sdk import actions
+
         # Execute an action from observe() results
         obs = session.observe()
         action = obs.space.first()  # Get first available action
         result = session.execute(action)
 
         # Execute a click action by element ID (use `obs.space.description` to checkout elements IDs)
-        result = session.execute({"type": "click", "id": "B1"})
+        result = session.execute(actions.Click(id="B1"))
 
         # Execute a form fill action
-        result = session.execute({"type": "fill", "id": "I1", "value": "user@example.com"})
+        result = session.execute(actions.Fill(id="I1", value="user@example.com"))
 
         # Execute browser navigation
-        result = session.execute({"type": "goto", "url": "https://example.com"})
+        result = session.execute(actions.Goto(url="https://example.com"))
         ```
 
         **Action Types:**
@@ -1066,23 +1254,15 @@ class RemoteSession(SyncResource):
         Instead of element IDs, you can use Playwright selectors to target elements:
 
         ```python
-        session.execute(type="fill", selector="internal:text=\"Email\"", value="test@example.com")
+        session.execute(actions.Fill(selector="internal:text=\"Email\"", value="test@example.com"))
         ```
 
-        This sythax also supports Xpath (e.g. `xpath=/html/body/div[3]/div/button[1]`) or CSS selectors (e.g. `css=button.submit`).
+        This syntax also supports Xpath (e.g. `xpath=/html/body/div[3]/div/button[1]`) or CSS selectors (e.g. `css=button.submit`).
         > Note that we strongly advice to use selectors over IDs for workflows automation because IDs are dependent on the page structure and can change over time.
 
-        **Agent Execution vs Workflow Execution:**
-
-        The `execute` method can be consumed by 2 different modalities:
-        - **Agent Execution:**
-            - This is the default mode and is used when you want to execute an action using an LLM in an agent loop.
-            - You should use the `session.execute(action, raise_on_failure=False)` to execute an action and not raise an exception if the action fails. You can use the execution result to provide feedback to the LLM for failure recovery.
-        - **Workflow Execution:**
-            - This is used when you want to execute a sequence of actions in a predefined order.
-            - You should use the `session.execute(action, raise_on_failure=True)` to execute an action and raise an exception if the action fails. This is similar as what playwright does internally.
 
         Args:
+            raise_on_failure: If true, will raise if we could not execute the action
             **data: Arbitrary keyword arguments matching the expected structure for a
                 step request.
 
